// wfracDlg.cpp : implementation file
//

#include "stdafx.h"
#include "wfrac.h"
#include "wfracDlg.h"

#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

//---------------------------------------------------------------------------

CWFracDlg::CWFracDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CWFracDlg::IDD, pParent)
	, m_Top(2.1)
	, m_Left(-2.1)
	, m_Right(2.1)
	, m_Bottom(-2.1)
	, m_sZr(_T("0"))
	, m_sZi(_T("0"))
	, m_sCr(_T("r"))
	, m_sCi(_T("i"))
	, m_bComputing(false)
	, m_Radius(1000)
	, m_MaxIter(96)
	, m_PotScale(200)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

//---------------------------------------------------------------------------

void CWFracDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_TOP, m_Top);
	DDX_Text(pDX, IDC_LEFT, m_Left);
	DDX_Text(pDX, IDC_RIGHT, m_Right);
	DDX_Text(pDX, IDC_BOTTOM, m_Bottom);
	DDX_Text(pDX, IDC_Z0X, m_sZr);
	DDX_Text(pDX, IDC_Z0Y, m_sZi);
	DDX_Text(pDX, IDC_C0X, m_sCr);
	DDX_Text(pDX, IDC_C0Y, m_sCi);
	DDX_Text(pDX, IDC_RADIUS, m_Radius);
	DDV_MinMaxDouble(pDX, m_Radius, 2, 1000000000);
	DDX_Text(pDX, IDC_MAXITER, m_MaxIter);
	DDV_MinMaxUInt(pDX, m_MaxIter, 4, 32767);
	DDX_Text(pDX, IDC_PSCALE, m_PotScale);
}

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CWFracDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

BOOL CWFracDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

//---------------------------------------------------------------------------

void CWFracDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

//---------------------------------------------------------------------------

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CWFracDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

//---------------------------------------------------------------------------

double QuadFrac(double zr, double zi, double cr, double ci,
                double radius, int maxiter)
// return potential of quadratic fractal (z=z*z+c)
// exit conditions: |z| > radius or itercount >= maxiter
{


  // make sure positive iteration count
  if (maxiter < 0)
    maxiter = 2;

  // we'll check against radius squared
  radius *= radius;

  // complex accumulator parts squared
  double zr2 = zr*zr, zi2 = zi*zi, zacc;

  // run iteration count
  int i = 1;
  do {

    // evaluate formula z*z + c
    zi = 2 * zr * zi + ci;
    zr = zr2 - zi2   + cr;

    // compute zr*zr and zi*zi
    zr2 = zr*zr;
    zi2 = zi*zi;

    zacc = zr2 + zi2;

  } while ((++i < maxiter) && (zacc < radius));

  const double ln2  = 0.693147180559945309;
  const double ln10 = 2.302585092994045680;
  return fabs(zr2) < 0.000001 ? 0 : 0.5 * (log(zacc) / ln10) / exp(i * ln2);
  // may need to be log10(zr2 + zi2) / pow(2.0, i + 1);

} // QuadFrac

//---------------------------------------------------------------------------

void CWFracDlg::StartComputing()
{

	if (!UpdateData(TRUE))
	{
		return;
	} // if

	m_sZr.MakeLower();
	m_sZi.MakeLower();
	m_sCr.MakeLower();
	m_sCi.MakeLower();

	double dZr  = atof(m_sZr);
	double dZi  = atof(m_sZi);
	double dCr  = atof(m_sCr);
	double dCi  = atof(m_sCi);

	int iZr = m_sZr == "r" ? 1 : m_sZr == "i" ? -1 : 0;
	int iZi = m_sZi == "r" ? 1 : m_sZi == "i" ? -1 : 0;
	int iCr = m_sCr == "r" ? 1 : m_sCr == "i" ? -1 : 0;
	int iCi = m_sCi == "r" ? 1 : m_sCi == "i" ? -1 : 0;

	CWnd* pWnd = GetDlgItem(IDC_IMAGE);
	if (pWnd == 0)
	{
		AfxMessageBox(
			"Something has gone horribly wrong; can't find the window!\n"
			"Please restart the program.");
		return;
	} // if

	CRect rect;
	pWnd->GetClientRect(rect);
	int w = rect.Width();
	int h = rect.Height();
	if (w == 0)
	{
		AfxMessageBox(
			"Somehow the window width is 0.\nThat's a problem.\n"
			"Please restart the program.");
		return;
	} // if
	if (h == 0)
	{
		AfxMessageBox(
			"Somehow the window width is 0.\nThat's a problem.\n"
			"Please restart the program.");
		return;
	} // if

	CDC* pDC = pWnd->GetDC();
	if (pDC == 0)
	{
		AfxMessageBox(
			"Failure, failure, failure!\nCan't get a device context!\n"
			"Please restart the program.");
		return;
	} // if

	DWORD nPix = 0;

	m_bComputing = true;

	double dx = (m_Right - m_Left) / double(w);
	double dy = (m_Top - m_Bottom) / double(h);

	for (int iy = 0; iy < h && m_bComputing; ++iy)
	{

		double y = m_Left + iy * dy;

		for (int ix = 0; ix < w && m_bComputing; ++ix)
		{

			double x = m_Bottom + ix * dx;

			double zr = iZr < 0 ? y : iZr > 0 ? x : dZr;
			double zi = iZi < 0 ? y : iZi > 0 ? x : dZi;
			double cr = iCr < 0 ? y : iCr > 0 ? x : dCr;
			double ci = iCi < 0 ? y : iCi > 0 ? x : dCi;

			double pot = QuadFrac(zr, zi, cr, ci, m_Radius, m_MaxIter);

			BYTE g = BYTE(255 * exp(-pot * m_PotScale));

			pDC->SetPixel(ix, iy, RGB(g, g, g));

		} // for

		if ((++nPix & 31) == 0)
		{
			MSG msg;
			while (PeekMessage(&msg, m_hWnd, 0, 0, FALSE) && m_bComputing)
			{
				if (!AfxGetApp()->PumpMessage())
				{
					m_bComputing = false;
				} // if
			} // while
		} // if

	} // for

	pWnd->ReleaseDC(pDC);

	m_bComputing = false;

} // CWFracDlg::StartComputing

//---------------------------------------------------------------------------

void CWFracDlg::StopComputing()
{
	m_bComputing = false;
} // CWFracDlg::StopComputing

//---------------------------------------------------------------------------

void CWFracDlg::OnOK()
{

	if (m_bComputing)
	{
		StopComputing();
	} // if
	else
	{
		StartComputing();
	} // else

} // CWFracDlg::OnOK

//---------------------------------------------------------------------------
